-----------------------
--- Module Creation ---
-----------------------
local core = LibStub("AceAddon-3.0"):GetAddon("tomQuest2")
local module = core:NewModule("notifications", "AceEvent-3.0", "AceHook-3.0", "LibSink-2.0")

-----------------------
--- Library loading ---
-----------------------
local LN = LibStub("AceLocale-3.0"):GetLocale("tomQuest2-Notifications")

--------------------------------------------
--- Copy global functions to local scope ---
--------------------------------------------
local format, gsub = string.format, string.gsub

-----------------
--- variables ---
-----------------
module.modName = LN["Quests Notifications"]
local prdb, options, output
local found = '^'..ERR_QUEST_ADD_FOUND_SII:gsub('(%%.)', '(.+)')..'$'
local item = '^'..ERR_QUEST_ADD_ITEM_SII:gsub('(%%.)', '(.+)')..'$'
local kill = '^'..ERR_QUEST_ADD_KILL_SII:gsub('(%%.)', '(.+)')..'$'
local complete = '^'..ERR_QUEST_OBJECTIVE_COMPLETE_S:gsub('(%%.)', '(.+)')..'$'

-----------------------
--- Default profile ---
-----------------------
local defaults = {
	profile = {
		playerObjectiveProgress = true,
		playerObjectiveComplete = true,
		playerQuestComplete = true,
		playerQuestFailed = true,
		partyObjectiveProgress = true,
		partyObjectiveComplete = true,
		partyQuestComplete = true,
		partyQuestFailed = true,
		partyQuestAccepted = true,
		partyQuestTurnedIn = true,
		partyQuestAbandonned = true,
		hideBlizzardNotification = true,
		objectiveNotificationWhenComplete = false,
		sinkOptions = {
		},
	},
}

----------------------
--- Initialization ---
----------------------
function module:OnInitialize()
	self.db = core.db:RegisterNamespace("notifications", defaults, "Default")
	prdb = self.db.profile
	self:SetSinkStorage(prdb.sinkOptions)
end

---------------
--- Options ---
---------------
function module:getOptions()
	if not options then
		options = {
			type = "group",
			name = module.modName,
			desc = module.modName,
			get = function(info) 
				return prdb[info[#info]]
			end,
			set = function(info, value)
				prdb[info[#info]] = value
			end,
			args = {
				hideBlizzardNotification = {
					name = LN["Toggle hide default blizzard quests notification"],
					type = "toggle",
					width = "full",
					order = 10,
				},
				playerheader = {
					type = "header",
					name = LN["Player Notifications"],
					order = 11,
				},
				playerObjectiveProgress = {
					name = LN["Toggle player quests objectives progress notification"],
					type = "toggle",
					width = "full",
					order = 12,
				},
				playerObjectiveComplete = {
					name = LN["Toggle player quests objectives complete notification"],
					type = "toggle",
					width = "full",
					order = 13,
				},
				playerQuestComplete = {
					name = LN["Toggle player quests complete notification"],
					type = "toggle",
					width = "full",
					order = 14,
				},
				playerQuestFailed = {
					name = LN["Toggle player quests failed notification"],
					type = "toggle",
					width = "full",
					order = 15,
				},
				partyheader = {
					type = "header",
					name = LN["Party Notifications"],
					order = 16,
				},
				partyObjectiveProgress = {
					name = LN["Toggle party quests objectives progress notification"],
					type = "toggle",
					width = "full",
					order = 17,
				},
				partyObjectiveComplete = {
					name = LN["Toggle party quests objectives complete notification"],
					type = "toggle",
					width = "full",
					order = 18,
				},
				partyQuestComplete = {
					name = LN["Toggle party quests complete notification"],
					type = "toggle",
					width = "full",
					order = 16,
				},
				partyQuestFailed = {
					name = LN["Toggle party quests failed notification"],
					type = "toggle",
					width = "full",
					order = 19,
				},
				partyQuestAccepted = {
					name = LN["Toggle party quest accepted notification"],
					type = "toggle",
					width = "full",
					order = 20,
				},
				partyQuestTurnedIn = {
					name = LN["Toggle party quest turned in notification"],
					type = "toggle",
					width = "full",
					order = 21,
				},
				partyQuestAbandonned = {
					name = LN["Toggle party quest abandonned notification"],
					type = "toggle",
					width = "full",
					order = 22,
				},
			},
		}
	end
	if not output then
		output = self:GetSinkAce3OptionsDataTable()
	end
	return options, output
end

function module:getOutputOptions()
	if not output then
		output = self:GetSinkAce3OptionsDataTable()
	end
	return output
end

function module:OnEnable()
	--self:RawHook("UIErrorsFrame_OnEvent", true)
	core.RegisterCallback(self, "playerQuestObjectiveProgress")
	core.RegisterCallback(self, "playerQuestComplete")
	core.RegisterCallback(self, "playerQuestFailed")
	core.RegisterCallback(self, "partyQuestObjectiveProgress")
	core.RegisterCallback(self, "partyQuestComplete")
	core.RegisterCallback(self, "partyQuestFailed")
	core.RegisterCallback(self, "partyQuestLost")
	core.RegisterCallback(self, "endPartyQuest")
end

function module:OnDisable()
	core.UnregisterAllCallbacks(self)
end

--[[function module:UIErrorsFrame_OnEvent(frame, event, msg, ...)
	if prdb.hideBlizzardNotification and event == "UI_INFO_MESSAGE" and
		(msg:match(found) or msg:match(item) or msg:match(kill) or msg:match(complete)) then return end
	return self.hooks.UIErrorsFrame_OnEvent(frame, event, msg, ...)
end]]--

local origiOnEvent = UIErrorsFrame:GetScript("OnEvent")
UIErrorsFrame:SetScript("OnEvent", function(self, event, message, ...)
	if prdb.hideBlizzardNotification and event == "UI_INFO_MESSAGE" and
		(message:match(found) or message:match(item) or message:match(kill) or message:match(complete)) then return end
	return origiOnEvent(self, event, message, ...)
end)

function module:playerQuestObjectiveProgress(event, firstRun, commHandler, uid, objective, got, nGot)
	if nGot == core.questsObjectives[uid][objective].nNeed and prdb.playerObjectiveComplete and not firstRun then
		if not prdb.objectiveNotificationWhenComplete and core.quests[uid].party[core.playerName].complete ~= 0 then return end
		module:Pour(format(LN["(%s) %s : finished"], core.quests[uid].title, core.questsObjectives[uid][objective].desc), 0, 1, 0)
	elseif prdb.playerObjectiveProgress and not firstRun and nGot > 0 then
		module:Pour(format("(%s) %s : %s/%s", core.quests[uid].title, core.questsObjectives[uid][objective].desc , got, core.questsObjectives[uid][objective].need), core:getColorFromCompletionRGB(nGot/core.questsObjectives[uid][objective].nNeed))
	end
end

function module:playerQuestComplete(event, firstRun, commHandler, uid)
	if prdb.playerQuestComplete and not firstRun then
		module:Pour(format(LN["(%s) : finished"], core.quests[uid].title), 0, 1, 0)
	end
end

function module:playerQuestFailed(event, firstRun, commHandler, uid)
	if prdb.playerQuestFailed and not firstRun then
		module:Pour(format(LN["(%s) : failed"], core.quests[uid].title), 1, 0, 0)
	end
end

function module:partyQuestObjectiveProgress(event, sender, uid, objective, got, nGot)
	if nGot == core.questsObjectives[uid][objective].nNeed and prdb.partyObjectiveComplete then
		if not prdb.objectiveNotificationWhenComplete and core.quests[uid].party[sender].complete ~= 0 then return end
		module:Pour(format(LN["(%s : %s) %s : finished"], sender, core.quests[uid].title, core.questsObjectives[uid][objective].desc), 0, 1, 0)
	elseif prdb.partyObjectiveProgress and nGot > 0 then
		module:Pour(format("(%s : %s) %s : %s/%s", sender, core.quests[uid].title, core.questsObjectives[uid][objective].desc , got, core.questsObjectives[uid][objective].need), core:getColorFromCompletionRGB(nGot/core.questsObjectives[uid][objective].nNeed))
	end
end

function module:partyQuestComplete(event, sender, uid)
	if prdb.partyQuestComplete then
		module:Pour(format(LN["(%s : %s) : finished"], sender, core.quests[uid].title), 0, 1, 0)
	end
end

function module:partyQuestFailed(event, sender, uid)
	if prdb.partyQuestFailed then
		module:Pour(format(LN["(%s : %s) : failed"], sender, core.quests[uid].title), 1, 0, 0)
	end
end

function module:partyQuestLost(event, sender, uid, abandoning, title)
	if abandoning and prdb.partyQuestAbandonned then
		module:Pour(format(LN["(%s : %s) : abandonned"], sender, title, 1, 0, 0))
	elseif not abandoning and prdb.partyQuestTurnedIn then
		module:Pour(format(LN["(%s : %s) : turned in"], sender, title, 0, 1, 0))
	end
end

function module:endPartyQuest(event, sender, uid, started, batch)
	if not batch and prdb.partyQuestAccepted then
		module:Pour(format(LN["(%s : %s) : accepted"], sender, core.quests[uid].title, 0, 1, 0))
	end
end